package cvi10;

import io.reactivex.subjects.PublishSubject;

import org.webbitserver.BaseWebSocketHandler;
import org.webbitserver.WebServer;
import org.webbitserver.WebServers;
import org.webbitserver.WebSocketConnection;
import org.webbitserver.handler.StaticFileHandler;

import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPubSub;

public class WebSocketServer extends BaseWebSocketHandler {
    private int connectionCount;

	protected PublishSubject<DrawMessage> drawMessages = PublishSubject.create();
	protected MessageSerde serde = new MessageSerde();
	
    public void onOpen(WebSocketConnection connection) {
    	System.out.println("Client accepted.");
        connectionCount++;
        
        // pass message from REDIS to websocket
        Jedis client = JedisConnectionFactory.newClient();
        client.subscribe(new JedisPubSub() {
			@Override
			public void onMessage(String channel, String message) {
				super.onMessage(channel, message);
				connection.send(message);
			}
		}, "drawing");
    }

    public void onClose(WebSocketConnection connection) {
        connectionCount--;
    }

    public void onMessage(WebSocketConnection connection, String message) {
    	drawMessages.onNext(serde.deserialize(message));
    }

    public PublishSubject<DrawMessage> getDrawMessages() {
		return drawMessages;
	}

	public static WebSocketServer createAndRunServer() {
    	WebSocketServer webSocket = new WebSocketServer();
        WebServer webServer = WebServers.createWebServer(8081)
                .add("/hellowebsocket", webSocket)
                .add(new StaticFileHandler("/web"));
        webServer.start();
        
        System.out.println("Websocket server running.");
        
        return webSocket;
    }
}